
/*

  netclip.h -- headerfile for netclip
  Copyright (C) 2017  Charlie Sale

  This file is part of netclip

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef DECLS_H
#define DECLS_H

#include <sys/queue.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/* Request macros */
#define NCR_CREAT "create"
#define NCR_READ  "read"
#define NCR_GET   "get"

/* Declaration of types */

/* clip_t -- represents a text clip */
typedef struct clip
{
  char        *buffer; // buffer that stores clip size
  int          idx;    // index of clip
  struct clip *left;   // cliping to the left
} clip_stack_t;

/* Declaration of functions */

/* Stack functions */

/* Creates a new clip:
   char* --> data to place inside clip
   clip_t* --> pointer to left clip
*/
clip_stack_t*
clip_stack_new(char*, clip_stack_t*);

/* clip_stack_free -- frees a stack */
void
clip_stack_free(clip_stack_t*);

/* clip_stack_push -- pushes a new clip onto the stack */
void
clip_stack_push(clip_stack_t*, clip_stack_t*);

/* Get the top item without popping the stack */
clip_stack_t*
clip_stack_get_top(clip_stack_t*);

/* clip_stack_pop -- pops a clip from the stack */
clip_stack_t*
clip_stack_pop(clip_stack_t*);

/* Networking functions */

typedef int netclip_sock_t; // just to make a more specific type

/* Set up networking connection */

// client specific
int
netclip_cli_init(const char*, int);

// server specific
int
netclip_serv_init(int);

/* Send a clip */
int
netclip_net_send(int, const char*, clip_stack_t*);

/* Recv a clip */
clip_stack_t*
netclip_net_recv(int, char*);

void
netclip_net_close(int);

/* The actual netclip API */

/* netclip functions */

#endif // DELCS_H
