

/*

  clip_stack.c -- defininition of clip stack routines
  Copyright (C) 2017  Charlie Sale

  This file is part of netclip

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include <netclip.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

clip_stack_t*
clip_stack_new(char *data, clip_stack_t *root)
{
  clip_stack_t *tmp;
  tmp->buffer = malloc (sizeof(data)); // allocate buffer to be data's size
  tmp->buffer = strdup (data); // dump string's data into the buffer
  tmp->idx    = root->idx++;
  tmp->left   = NULL;
  root->left  = tmp;

  return tmp;
}

void
clip_stack_push(clip_stack_t *child, clip_stack_t *root)
{
  if (root->left)
    return clip_stack_push (child, root->left);
  else
    root->left = child;
}

clip_stack_t*
clip_stack_get_top(clip_stack_t *root)
{
  if (root->left)
    return clip_stack_pop (root->left);
  else
    return root;
}

clip_stack_t*
clip_stack_pop(clip_stack_t *root)
{
  // like the other, but we need to look two in front to pop from stack
  if (root->left->left) {
    return clip_stack_pop (root->left);
  } else {

    clip_stack_t *tmp = root->left;
    root->left = NULL;

    return tmp;

  }
}

void
clip_stack_free(clip_stack_t *root)
{
  if (root->left)
    {
      clip_stack_t tmp = &root->left;
      free (root);
      clip_stack_free (&tmp);
    }
}
