

/*

  net.c -- handles some networking protocols
  Copyright (C) 2017  Charlie Sale

  This file is part of netclip

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <netclip.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* begin */

int
netclip_cli_init(const char *host, int port)
{
  int fd; // create a file descriptor

  struct addrinfo  hints;    // used for connecting
  struct addrinfo *servinfo; // info about server to connect to

  memset(&hints, 0, sizeof hints);
  hints.ai_family   = AF_UNSPEC;   // unspecified family
  hints.ai_socktype = SOCK_STREAM; // socket stream socket
  hints.ai_flags    = AI_PASSIVE;  // fill in ip for me

  // quick little conv
  char number[10];
  itoa(port, number, 20);

  // get connection info
  int ret = getaddrinfo(host, number, &hints, &servinfo);
  if (ret != 0)
    {
      return -1;
    }

  // create socket with address info
  fd = socket(servinfo->ai_family, servinfo->ai_socktype, servinfo->ai_protocol);
  if (fd < 0)
    {
      return -1;
    }

  ret = connect(fd, servinfo->ai_addr, servinfo->ai_addrlen);
  if (ret != 0)
    return -1;

  // socket is connected...
  return fd;
}

int
netclip_serv_init(int port)
{
  int fd;

  struct addrinfo  hints; // used for connecting
  struct addrinfo *info;  // info about address

  memset(&hints, 0, sizeof hints);

  hints.ai_family = AF_UNSPEC;
  hints.ai_socktype = SOCK_STREAM;
  hints.ai_flags = AF_PASSIVE; // autoset IP

  int status;

  char num[10];
  itoa(port, num, 20);

  status = getaddrinfo(NULL, num, &hints, &info);
  if (status != 0)
    return -1;

  fd = socket(info->ai_family, info->ai_socktype, info->ai_protocol);
  if (fd < 0)
    return -1;

  status = bind(fd, info->ai_addr, info->ai_addrlen);
  if (status != 0)
    return -1;

  // bound
  return fd;
}

int
netclip_net_send(int fd, const char *req, clip_stack_t *clip)
{
  int status;

  // first, send request so server knows what to do...
  status = send(fd, req, sizeof req, 0);
  if (status < 0)
    return -1;

  // send data
  status = send(fd, clip->buffer, sizeof clip->buffer, 0);
  if (status < 0)
    return -1;

  return status;
}

char *
netclip_net_recv(int fd, char *request)
{
  int status;

  char req[512];
  char buf[512];

  status = recv(fd, req, sizeof req, 0);
  if (status < 0)
    return NULL;

  // read request
  request = strdup(req);

  status = recv(fd, buf, sizeof buf, 0);
  if (status < 0)
    return NULL;

  return buf;
}

void
netclip_net_close(int fd)
{
  close(fd);
}
